@@ -113,9 +113,17 @@ static const char *introspection_xml =
     "            <annotation name=\"org.freedesktop.DBus.Property.EmitsChangedSignal\" value=\"true\"/>"
     "        </property>"
 
-    "        <property name=\"displayedLength\" type=\"u\" access=\"read\" />"
-    "        <property name=\"historyLength\" type=\"u\" access=\"read\" />"
-    "        <property name=\"waitingLength\" type=\"u\" access=\"read\" />"
+    "        <property name=\"displayedLength\" type=\"u\" access=\"read\">"
+    "            <annotation name=\"org.freedesktop.DBus.Property.EmitsChangedSignal\" value=\"true\"/>"
+    "        </property>"
+    "        <property name=\"historyLength\" type=\"u\" access=\"read\">"
+    "            <annotation name=\"org.freedesktop.DBus.Property.EmitsChangedSignal\" value=\"true\"/>"
+    "        </property>"
+
+    "        <property name=\"waitingLength\" type=\"u\" access=\"read\">"
+    "            <annotation name=\"org.freedesktop.DBus.Property.EmitsChangedSignal\" value=\"true\"/>"
+    "        </property>"
+
 
     "        <signal name=\"NotificationHistoryRemoved\">"
     "            <arg name=\"id\"         type=\"u\"/>"
@@ -628,16 +636,16 @@ static void dbus_cb_dunst_RuleEnable(GDBusConnection *connection,
         }
 
         struct rule *target_rule = get_rule(name);
-        g_free(name);
-
         if (target_rule == NULL) {
                 g_dbus_method_invocation_return_error(invocation,
                         G_DBUS_ERROR,
                         G_DBUS_ERROR_INVALID_ARGS,
                         "There is no rule named \"%s\"",
                         name);
+                g_free(name);
                 return;
         }
+        g_free(name);
 
         if (state == 0)
                 target_rule->enabled = false;
@@ -896,7 +904,6 @@ static struct notification *dbus_message_to_notification(const gchar *sender, GV
                         }
                 }
 
-
                 gradient_pattern(grad);
 
                 notification_keep_original(n);
@@ -994,6 +1001,39 @@ void signal_length_propertieschanged(void)
         g_clear_pointer(&invalidated_builder, g_variant_builder_unref);
 }
 
+void signal_paused_propertieschanged(void)
+{
+        if(!dbus_conn)
+                return;
+
+        struct dunst_status status = dunst_status_get();
+
+        GVariantBuilder *builder = g_variant_builder_new(G_VARIANT_TYPE_VARDICT);
+        GVariantBuilder *invalidated_builder = g_variant_builder_new(G_VARIANT_TYPE_STRING_ARRAY);
+
+        g_variant_builder_add(builder,
+                              "{sv}",
+                              "paused", g_variant_new_boolean(status.pause_level != 0));
+        g_variant_builder_add(builder,
+                              "{sv}",
+                              "pauseLevel", g_variant_new_uint32(status.pause_level));
+
+        g_dbus_connection_emit_signal(dbus_conn,
+                                      NULL,
+                                      FDN_PATH,
+                                      PROPERTIES_IFAC,
+                                      "PropertiesChanged",
+                                      g_variant_new("(sa{sv}as)",
+                                                    DUNST_IFAC,
+                                                    builder,
+                                                    invalidated_builder),
+                                      NULL);
+
+        g_clear_pointer(&builder, g_variant_builder_unref);
+        g_clear_pointer(&invalidated_builder, g_variant_builder_unref);
+
+}
+
 static void dbus_cb_Notify(
                 GDBusConnection *connection,
                 const gchar *sender,
@@ -1200,27 +1240,8 @@ gboolean dbus_cb_dunst_Properties_Set(GDBusConnection *connection,
                 dunst_status_int(S_PAUSE_LEVEL, targetPauseLevel);
                 wake_up();
 
-                GVariantBuilder *builder = g_variant_builder_new(G_VARIANT_TYPE_VARDICT);
-                GVariantBuilder *invalidated_builder = g_variant_builder_new(G_VARIANT_TYPE_STRING_ARRAY);
-                g_variant_builder_add(builder,
-                                      "{sv}",
-                                      "paused", g_variant_new_boolean(targetPauseLevel != 0));
-                g_variant_builder_add(builder,
-                                      "{sv}",
-                                      "pauseLevel", g_variant_new_uint32(targetPauseLevel));
-                g_dbus_connection_emit_signal(connection,
-                                              NULL,
-                                              object_path,
-                                              PROPERTIES_IFAC,
-                                              "PropertiesChanged",
-                                              g_variant_new("(sa{sv}as)",
-                                                            interface_name,
-                                                            builder,
-                                                            invalidated_builder),
-                                              NULL);
+                signal_paused_propertieschanged();
 
-                g_clear_pointer(&builder, g_variant_builder_unref);
-                g_clear_pointer(&invalidated_builder, g_variant_builder_unref);
                 return true;
         }
 
@@ -1511,5 +1532,3 @@ void dbus_teardown(int owner_id)
         g_bus_unown_name(owner_id);
         dbus_conn = NULL;
 }
-
-/* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
