@@ -39,6 +39,9 @@ void dunst_status(const enum dunst_status_field field,
         case S_IDLE:
                 status.idle = value;
                 break;
+        case S_MOUSE_OVER:
+                status.mouse_over = value;
+                break;
         default:
                 LOG_E("Invalid %s enum value in %s:%d for bool type", "dunst_status", __FILE__, __LINE__);
                 break;
@@ -184,6 +187,8 @@ gboolean pause_signal(gpointer data)
         dunst_status_int(S_PAUSE_LEVEL, MAX_PAUSE_LEVEL);
         wake_up();
 
+        signal_paused_propertieschanged();
+
         return G_SOURCE_CONTINUE;
 }
 
@@ -194,6 +199,8 @@ gboolean unpause_signal(gpointer data)
         dunst_status_int(S_PAUSE_LEVEL, 0);
         wake_up();
 
+        signal_paused_propertieschanged();
+
         return G_SOURCE_CONTINUE;
 }
 
@@ -370,5 +377,3 @@ void print_version(void)
 #endif
         exit(EXIT_SUCCESS);
 }
-
-/* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
