@@ -65,6 +65,7 @@ struct notification {
         cairo_surface_t *icon;         /**< The raw cached icon data used to draw */
         char *icon_id;           /**< Plain icon information, which acts as the icon's id.
                                       May be a hash for a raw icon or a name/path for a regular app icon. */
+        gint64 icon_time;        /**< Time of reception of the icon (or opening of the file in case of a path) */
         char *iconname;          /**< plain icon information (may be a path or just a name) as recieved from dbus.
                                    Use this to compare the icon name with rules. May also be modified by rules.*/
         char *icon_path;         /**< Full path to the notification's icon. */
@@ -106,6 +107,7 @@ struct notification {
         enum behavior_fullscreen fullscreen; //!< The instruction what to do with it, when desktop enters fullscreen
         bool script_run;        /**< Has the script been executed already? */
         guint8 marked_for_closure;
+        guint8 marked_for_removal; /**< If set, the notification is marked for removal in history */
         bool word_wrap;
         PangoEllipsizeMode ellipsize;
         PangoAlignment alignment;
@@ -274,4 +276,3 @@ const char *enum_to_string_fullscreen(enum behavior_fullscreen in);
 void notification_keep_original(struct notification *n);
 
 #endif
-/* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
