@@ -178,6 +178,7 @@ struct settings {
         int notification_limit;
         int gap_size;
         int default_pause_level;
+        bool pause_on_mouse_over;
 };
 
 extern struct settings settings;
@@ -188,4 +189,3 @@ void load_settings(char **const config_paths);
 void settings_free(struct settings *s);
 
 #endif
-/* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
