@@ -312,6 +312,7 @@ static const struct string_to_enum_def mouse_action_enum_data[] = {
         {"context",        MOUSE_CONTEXT },
         {"context_all",    MOUSE_CONTEXT_ALL },
         {"open_url",       MOUSE_OPEN_URL },
+        {"remove_current", MOUSE_REMOVE_CURRENT },
         ENUM_END,
 };
 
@@ -1220,6 +1221,36 @@ static const struct setting allowed_settings[] = {
                 .parser = string_parse_bool,
                 .parser_data = boolean_enum_data,
         },
+        {
+                .name = "gap_size",
+                .section = "global",
+                .description = "Size of gap between notifications",
+                .type = TYPE_INT,
+                .default_value = "0",
+                .value = &settings.gap_size,
+                .parser = NULL,
+                .parser_data = NULL,
+        },
+        {
+                .name = "default_pause_level",
+                .section = "global",
+                .description = "Start dunst with a this pause level set.",
+                .type = TYPE_INT,
+                .default_value = "0",
+                .value = &settings.default_pause_level,
+                .parser = NULL,
+                .parser_data = NULL,
+        },
+        {
+                .name = "pause_on_mouse_over",
+                .section = "experimental",
+                .description = "Pause notification timeout when mouse is over the notification window. Only works on Wayland.",
+                .type = TYPE_CUSTOM,
+                .default_value = "false",
+                .value = &settings.pause_on_mouse_over,
+                .parser = string_parse_bool,
+                .parser_data = boolean_enum_data,
+        },
         // manual extractions below
         {
                 .name = "follow",
@@ -1286,7 +1317,7 @@ static const struct setting allowed_settings[] = {
                 .section = "global",
                 .description = "Action of middle click event",
                 .type = TYPE_LIST,
-                .default_value = "do_action, close_current",
+                .default_value = "do_action, remove_current",
                 .value = &settings.mouse_middle_click,
                 .parser = NULL,
                 .parser_data = GINT_TO_POINTER(MOUSE_LIST),
@@ -1598,16 +1629,6 @@ static const struct setting allowed_settings[] = {
                 .parser = NULL,
                 .parser_data = NULL,
         },
-        {
-                .name = "gap_size",
-                .section = "global",
-                .description = "Size of gap between notifications",
-                .type = TYPE_INT,
-                .default_value = "0",
-                .value = &settings.gap_size,
-                .parser = NULL,
-                .parser_data = NULL,
-        },
 };
+
 #endif
-/* vim: set ft=c tabstop=8 shiftwidth=8 expandtab textwidth=0: */
